
class EventParameterIsNotValueObjectError(Exception):
    pass


class MutatorNotFoundError(Exception):
    pass


class SingleMutatorBrokenError(Exception):
    pass


class ValueObjectIsNotSerializable(Exception):
    pass
